<?php

/**
 * @file
 * Contains Drupal\encrypt_kms\Plugin\KeyProvider\AwsKmsKeyProvider.
 */

namespace Drupal\encrypt_kms\Plugin\KeyProvider;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\encrypt\EncryptionProfileManagerInterface;
use Drupal\encrypt\EncryptServiceInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyPluginFormInterface;
use Drupal\key\Plugin\KeyProviderBase;
use Drupal\key\Plugin\KeyProviderSettableValueInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Adds a key provider that allows a key to be stored in AWS KMS.
 *
 * @KeyProvider(
 *   id = "aws_kms",
 *   label = "AWS KMS",
 *   description = @Translation("The AWS KMS key provider will store the encrypted key locally."),
 *   storage_method = "aws_kms", key_value = {
 *     "accepted" = TRUE,
 *     "required" = TRUE
 *   }
 * )
 */
class AwsKmsKeyProvider extends KeyProviderBase implements KeyProviderSettableValueInterface, KeyPluginFormInterface {

  /**
   * The KMS client.
   *
   * @var \Drupal\encrypt\EncryptService
   */
  protected $encryptionService;

  /**
   * The Encryption profile manager.
   *
   * @var \Drupal\encrypt\EncryptionProfileManager
   */
  protected $profileManager;

  /**
   * The Encrypt KMS secret storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $secretStorage;


  /**
   * Construct the AwsKmsKeyProvider object.
   *
   * @param array $configuration
   *   The configuration of the plugin.
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager;
   *   The entity type manager.
   * @param \Drupal\encrypt\EncryptServiceInterface $encryptionService
   *   The Encryption service.
   * @param \Drupal\encrypt\EncryptionProfileManagerInterface $profileManager
   *   The Encryption Profile Manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EncryptServiceInterface $encryptionService, EncryptionProfileManagerInterface $profileManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->encryptionService = $encryptionService;
    $this->profileManager = $profileManager;
    $this->secretStorage = $entity_type_manager->getStorage('aws_kms_secret');
  }

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container.
   * @param array $configuration
   *   The configuration of the plugin.
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   *
   * @return static
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('encryption'),
      $container->get('encrypt.encryption_profile.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'client_master_profile' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    if (!class_exists('\Aws\Kms\KmsClient')) {
      $form['need_library'] = [
        '#prefix' => '<p>',
        '#markup' => $this->t('AWS KMS PHP library is not correctly installed.'),
        '#suffix' => '</p>',
      ];
    }

    $options = [];
    $encryptionProfiles = $this->profileManager->getEncryptionProfilesByEncryptionMethod("aws_kms");

    if ($encryptionProfiles) {
      foreach ($encryptionProfiles as $encryptionProfile) {
        $options[$encryptionProfile->id()] = $encryptionProfile->label();
      }
    }

    $form['client_master_profile'] = [
      '#type' => 'select',
      '#title' => $this->t('Client Master Profile'),
      '#required' => TRUE,
      '#options' => $options,
      '#description' => $this->t('Choose an available profile, using the Amazon KMS encryption method.'),
      '#default_value' => $this->configuration['client_master_profile'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyValue(KeyInterface $key) {
    $encryptedKey = $this->secretStorage->load($key->id());
    $encryptionProfile = $this->profileManager->getEncryptionProfile($this->configuration['client_master_profile']);

    return $this->encryptionService->decrypt($encryptedKey->getValue(), $encryptionProfile);
  }

  /**
   * {@inheritdoc}
   */
  public function setKeyValue(KeyInterface $key, $key_value) {
    $encryptionProfile = $this->profileManager->getEncryptionProfile($this->configuration['client_master_profile']);
    $cipherText = $this->encryptionService->encrypt($key_value, $encryptionProfile);

    $secret = $this->secretStorage->create([
      'id' => $key->id(),
      'value' => $cipherText,
    ]);
    $ret_val = $this->secretStorage->save($secret);

    return isset($ret_val);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteKeyValue(KeyInterface $key) {
    $encryptedKey = $this->secretStorage->load($key->id());
    $this->secretStorage->delete($encryptedKey);

    return TRUE;
  }

}
