<?php

namespace Drupal\encrypt_kms\Plugin\KeyType;

use Drupal\Core\Form\FormStateInterface;
use Drupal\encrypt\EncryptionProfileManagerInterface;
use Drupal\key\Plugin\KeyPluginFormInterface;
use Drupal\key\Plugin\KeyType\EncryptionKeyType;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a KMS data key type
 *
 * @KeyType(
 *   id = "aws_kms_data",
 *   label = @Translation("KMS Data Key"),
 *   description = @Translation("The data key generated by the client master key."),
 *   group = "encryption",
 *   key_value = {
 *     "plugin" = "generate"
 *   }
 * )
 */
class KmsDataKeyType extends EncryptionKeyType implements KeyPluginFormInterface {

  /**
   * The Encryption profile manager.
   *
   * @var \Drupal\encrypt\EncryptionProfileManagerInterface
   */
  protected $profileManager;


  /**
   * Constructs the KmsDataKeyType
   *
   * @param array $configuration
   *   The configuration of the plugin.
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   * @param \Drupal\encrypt\EncryptionProfileManagerInterface $profileManager
   *   The Encryption Profile Manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EncryptionProfileManagerInterface $profileManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->profileManager = $profileManager;
  }

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container.
   * @param array $configuration
   *   The configuration of the plugin.
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   *
   * @return static
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('encrypt.encryption_profile.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
        'client_master_profile' => NULL,
      ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $form = parent::buildConfigurationForm($form, $form_state);

    if (!class_exists('\Aws\Kms\KmsClient')) {
      $form['need_library'] = [
        '#prefix' => '<p>',
        '#markup' => $this->t('AWS KMS PHP library is not correctly installed.'),
        '#suffix' => '</p>',
      ];
    }
    $options = [];
    $encryptionProfiles = $this->profileManager->getEncryptionProfilesByEncryptionMethod("aws_kms");

    if ($encryptionProfiles) {
      foreach ($encryptionProfiles as $encryptionProfile) {
        $options[$encryptionProfile->id()] = $encryptionProfile->label();
      }
    }

    $form['client_master_profile'] = [
      '#type' => 'select',
      '#title' => $this->t('Client Master Profile'),
      '#required' => TRUE,
      '#options' => $options,
      '#description' => $this->t('Choose an available profile, using the Amazon KMS encryption method.'),
      '#default_value' => $this->configuration['client_master_profile'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public static function generateKeyValue(array $configuration) {
    $profileManager = \Drupal::service('encrypt.encryption_profile.manager');
    $kmsClient = \Drupal::service('encrypt_kms.kms_client');

    $encryptionProfile = $profileManager->getEncryptionProfile($configuration['client_master_profile']);
    $key = $encryptionProfile->getEncryptionKey();

    if ($configuration['key_size'] == "128" || $configuration['key_size'] == "256") {
      $result = $kmsClient->generateDataKey([
        'KeyId' => $key->getKeyValue(),
        'KeySpec' => 'AES_' . $configuration['key_size'],
      ]);
    }
    else {
      $keySizeBytes = intval($configuration['key_size_other_value']);

      $result = $kmsClient->generateDataKey([
        'KeyId' => $key->getKeyValue(),
        'NumberOfBytes' => ($keySizeBytes / 8),
      ]);
    }

    return $result['Plaintext'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateKeyValue(array $form, FormStateInterface $form_state, $key_value) {
    // Validation of the key value is optional.
  }

}
